package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.basedata.BaseDataServiceHelper;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.HRHisBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.List;

/**
 * 职位族数据库操作类
 *
 * @author rimwood
 * @since 2021-12-08 17:37:42
 */
public class JobFamilyRepository {
    private static class JobFamilyInstance {
        private static JobFamilyRepository INSTANCE = new JobFamilyRepository();
    }

    public static JobFamilyRepository getInstance() {
        return JobFamilyInstance.INSTANCE;
    }

    /**
     * 查询受控业务单元下的职位族
     * @param orgId 受控业务单元id
     * @return 职位族数组
     */
    public DynamicObject[] queryJobFamilyByOrgId(Long orgId){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter orgFilter = BaseDataServiceHelper.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBFAMILIY, orgId);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in,
                new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED});
        return serviceHelper.query("id,name,number,jobseq",
                new QFilter[]{currentFilter,orgFilter,enableFilter,initStatusFilter},"number asc,id desc");
    }

    /**
     * 根据职位族id集合查询职位族
     * @param ids 职位族id集合
     * @return 职位族数组
     */
    public DynamicObject[] queryJobFamilyByIds(List<Long> ids){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter idsFilter = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,jobseq,createorg,enable",new QFilter[]{currentFilter,idsFilter,enableFilter},"number asc,id desc");
    }

    /**
     * 根据职位族id集合查询职位族,忽略状态
     * @param ids 职位族id集合
     * @return 职位族数组
     */
    public DynamicObject[] queryJobFamilyByIdsIgnoreEnable(List<Long> ids){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter idsFilter = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,jobseq,createorg,enable",new QFilter[]{currentFilter,idsFilter},"number asc,id desc");
    }

    /**
     * 根据职位族boid查询历史版本
     * @param boidList
     * @return
     */
    public DynamicObject[] getJobFamilyHisByBoId(Long[] boidList) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter boidQFilter = new QFilter(HisFieldNameConstants.BOID, QCP.in, boidList);
        QFilter iscurrentversionQfilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QCP.equals, "0");
        QFilter datastatusQFilter = new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QCP.in,
                new String[]{HRHisBaseConstants.EFFECTSTATUS_SCHEDULEEFFECT, HRHisBaseConstants.EFFECTSTATUS_EFFECTING,
                        HRHisBaseConstants.EFFECTSTATUS_INVALID});
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.equals, HBJMConstants.INIT_STATUS_COMPLETE);
        QFilter qFilter = boidQFilter.and(datastatusQFilter).and(iscurrentversionQfilter).and(initStatusFilter);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{qFilter});
    }
}
