package kd.hrmp.hbjm.business.domain.repository;

import com.alibaba.fastjson.JSONArray;
import kd.bos.basedata.cache.BaseDataCtrlCache;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobScmConstants;
import kd.hrmp.hbjm.common.util.PermOrgQFilterUtil;

import java.util.List;

/**
 * @author rd_yan_h
 * @desc 职位体系方案查询相关的Repository
 * @create 2021/12/06 10:14
 **/
public class JobScmRepository {
    /**
     * 日志
     */
    private static final Log logger = LogFactory.getLog(JobScmRepository.class);

    private static class JobScmInstance {
        private static JobScmRepository INSTANCE = new JobScmRepository();
    }

    public static JobScmRepository getInstance() {
        return JobScmInstance.INSTANCE;
    }

    /**
     * 根据用户id查询对应权限下职位体系方案id
     *
     * @param userId 用户id
     * @param orgId 业务单元id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobScm(Long userId, List<Long> orgId, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSCM);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        }
        if (userId != 0L) {
            QFilter permOrgFilter;
            List<Long> newOrgId = JSONArray.parseArray(orgId.toString(),Long.class);
            permOrgFilter = PermOrgQFilterUtil.assembleFilterByOr(HBJMConstants.PAGE_JOBSCM, newOrgId);
            if (permOrgFilter == null) {
                permOrgFilter = PermOrgQFilterUtil.getPermOrgs(userId, HBJMConstants.APP_HJM, HBJMConstants.PAGE_JOBSCM);
            }
            if (permOrgFilter != null) {
                enableFilter.and(permOrgFilter);
            } else {
                logger.info("JobScmRepository#JobScm'perm is null");
            }
        }
        return serviceHelper.queryOriginalArray("id,name,enable", new QFilter[]{enableFilter}, orderCondition);
    }

    /**
     * 根据 id查询
     *
     * @param id 职位体系id
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject queryBaseInfoById(Long id) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter[] filters = new QFilter[1];
        filters[0] = new QFilter(HRBaseConstants.ID, "=", id);
        return serviceHelper.loadDynamicObject(filters);
    }

    /**
     * 根据 ids查询
     *
     * @param ids 职位体系id
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject[] queryJobScmByIds(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter[] filters = new QFilter[1];
        filters[0] = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        return serviceHelper.loadDynamicObjectArray(filters);
    }

    /**
     * 根据 Number查询
     *
     * @param Number 职位体系编码
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject queryBaseInfoByNumber(String Number) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter[] filters = new QFilter[1];
        filters[0] = new QFilter(HRBaseConstants.NUMBER, "=", Number);
        return serviceHelper.queryOne(filters);
    }

    /**
     * 根据 Number查询
     *
     * @param Number 职位体系编码
     * @param permQfilter 权限Qfilter
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject queryBaseInfoByNumberAndPerm(String Number,QFilter permQfilter) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter[] filters = new QFilter[2];
        filters[0] = new QFilter(HRBaseConstants.NUMBER, "=", Number);
        filters[1] = permQfilter;
        return serviceHelper.queryOne(filters);
    }

    /**
     * 根据 Number查询
     *
     * @param Number 职位体系编码
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject queryBaseInfoByNumberAndOrg(String Number,Long orgId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter numberFilter = new QFilter(HRBaseConstants.NUMBER, "=", Number);
        QFilter orgFilter = new QFilter(HRBaseConstants.ORG, "=", orgId);
        return serviceHelper.queryOne(new QFilter[]{numberFilter,orgFilter});
    }

    /**
     * 根据 Name查询
     *
     * @param Name 职位体系编码名称
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject queryBaseInfoByName(String Name) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter[] filters = new QFilter[1];
        filters[0] = new QFilter(HRBaseConstants.NAME, "=", Name);
        return serviceHelper.queryOne(filters);
    }

    /**
     * 根据 Name查询
     *
     * @param Name 职位体系编码名称
     * @return DynamicObject 职位体系方案对象
     */
    public DynamicObject queryBaseInfoByNameAndOrg(String Name,Long orgId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        QFilter nameFilter = new QFilter(HRBaseConstants.NAME, "=", Name);
        QFilter orgFilter = new QFilter(HRBaseConstants.ORG, "=", orgId);
        return serviceHelper.queryOne(new QFilter[]{nameFilter,orgFilter});
    }

    /**
     * 保存职位体系方案基本信息
     * @param jobScmDy 基本信息对象
     */
    public void saveBaseInfo(DynamicObject jobScmDy) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        serviceHelper.saveOne(jobScmDy);
        // 清除_U表针对当前业务单元的缓存，不然列表显示不了数据
        BaseDataCtrlCache.clearBaseDataUseRange(JobScmConstants.PAGE_HBJM_JOBSCMHR, jobScmDy.getLong("org.id"));
    }

    /**
     * 修改职位体系方案基本信息
     * @param jobScmDy 基本信息对象
     */
    public void updateBaseInfo(DynamicObject jobScmDy) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(JobScmConstants.PAGE_HBJM_JOBSCMHR);
        serviceHelper.updateDataOne(jobScmDy);
        // 清除_U表针对当前业务单元的缓存，不然列表显示不了数据
        BaseDataCtrlCache.clearBaseDataUseRange(JobScmConstants.PAGE_HBJM_JOBSCMHR, jobScmDy.getLong("org.id"));
    }
}
