package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.basedata.service.BaseDataServiceImpl;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.permission.api.HasPermOrgResult;
import kd.bos.servicehelper.basedata.BaseDataServiceHelper;
import kd.bos.servicehelper.model.PermissionStatus;
import kd.bos.servicehelper.permission.PermissionServiceHelper;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.HRHisBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.List;
import java.util.Objects;

/**
 * 职位序列数据库操作类
 *
 * @author rimwood
 * @since 2021-12-08 17:37:42
 */
public class JobSeqRepository {
    private static class JobSeqInstance {
        private static JobSeqRepository INSTANCE = new JobSeqRepository();
    }

    public static JobSeqRepository getInstance() {
        return JobSeqInstance.INSTANCE;
    }

    /**
     * 查询受控业务单元下的职位序列
     *
     * @param orgId 受控业务单元id
     * @return 职位序列数组
     */
    public DynamicObject[] queryJobSeqByOrgId(Long orgId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter orgFilter = BaseDataServiceHelper.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBSEQ, orgId);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in,
                new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED});
        return serviceHelper.query("id,name,number",
                new QFilter[]{orgFilter, enableFilter, currentFilter, initStatusFilter}, "number asc,id desc");
    }

    /**
     * 根据职位序列id集合查询职位序列
     *
     * @param ids id集合
     * @return 职位序列数组
     */
    public DynamicObject[] queryJobSeqByIds(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter idsFilter = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,createorg,standardjobseq,enable", new QFilter[]{currentFilter, enableFilter, idsFilter}, "number asc,id desc");
    }

    /**
     * 根据职位序列id集合查询职位序列,忽略状态
     *
     * @param ids id集合
     * @return 职位序列数组
     */
    public DynamicObject[] queryJobSeqByIdsIgnoreEnable(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter idsFilter = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,createorg,standardjobseq,enable", new QFilter[]{currentFilter, idsFilter}, "number asc,id desc");
    }

    /**
     * 根据用户id查询对应权限下职位序列id
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobSeqByPerm(Long userId, Boolean isShowDisable) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        }
        HasPermOrgResult orgPerm = PermissionServiceHelper.getAllPermOrgs(userId, HBJMConstants.APP_HJM,
                HBJMConstants.PAGE_HBJM_JOBSEQ, PermissionStatus.View);
        if (!Objects.isNull(orgPerm) && !orgPerm.hasAllOrgPerm()) {
            List<Long> perms = orgPerm.getHasPermOrgs();
            if (Objects.isNull(perms) || perms.isEmpty()) {
                return null;
            }
            QFilter permFilter = new BaseDataServiceImpl().getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBSEQ, perms, true);
            enableFilter.and(permFilter);
        }
        return serviceHelper.queryOriginalArray(HRBaseConstants.ID, new QFilter[]{enableFilter});
    }

    /**
     * 根据职位序列boid查询历史版本
     * @param boidList
     * @return
     */
    public DynamicObject[] getJobSeqHisByBoId(Long[] boidList) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBSEQ);
        QFilter boidQFilter = new QFilter(HisFieldNameConstants.BOID, QCP.in, boidList);
        QFilter iscurrentversionQfilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QCP.equals, "0");
        QFilter datastatusQFilter = new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QCP.in,
                new String[]{HRHisBaseConstants.EFFECTSTATUS_SCHEDULEEFFECT, HRHisBaseConstants.EFFECTSTATUS_EFFECTING,
                        HRHisBaseConstants.EFFECTSTATUS_INVALID});
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.equals, HBJMConstants.INIT_STATUS_COMPLETE);
        QFilter qFilter = boidQFilter.and(datastatusQFilter).and(iscurrentversionQfilter).and(initStatusFilter);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{qFilter});
    }
}
