package kd.hrmp.hbjm.business.init;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hrmp.hbjm.business.domain.service.ChangeEventService;
import kd.hrmp.hbjm.business.domain.service.impl.ChangeEventServiceImpl;
import kd.hrmp.hbjm.common.constants.HBJMConstants;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 职位初始化处理服务
 *
 * @author rd_ningjie
 * @since 2022年03月12日 12:33:06
 */
public class JobInitComsumerService extends HisBaseDataComsumerService {

    private static Log LOG = LogFactory.getLog(JobInitComsumerService.class);

    @Override
    protected void dealDynData(List<DynamicObject> dyns, List<Map<String, Object>> datamaps) {
        super.dealDynData(dyns, datamaps);

        LOG.info("JobInitComsumerService dealDynData start", dyns);
        if (!ObjectUtils.isEmpty(dyns) && !ObjectUtils.isEmpty(datamaps)) {
            for (DynamicObject dyna : dyns) {
                //职位序列，如果职位序列为空，则可以从职位族上获取
                DynamicObject jobSeq = dyna.getDynamicObject(HBJMConstants.JOB_SEQ);
                if (ObjectUtils.isEmpty(jobSeq)) {
                    DynamicObject jobFamily = dyna.getDynamicObject(HBJMConstants.JOBFAMILY);
                    if (!ObjectUtils.isEmpty(jobFamily)) {
                        dyna.set(HBJMConstants.JOB_SEQ, jobFamily.get(HBJMConstants.JOB_SEQ));
                    }
                }
            }
        }
        LOG.info("JobInitComsumerService dealDynData end");
    }

    @Override
    public void rollback() {
        String entityCode = this.getInitInParam().getEntityNumber();
        HRBaseServiceHelper hrBaseServiceHelper = new HRBaseServiceHelper(entityCode);
        DynamicObject[] dynamicObjects = hrBaseServiceHelper.queryOriginalArray(HRBaseConstants.ID + "," + HisFieldNameConstants.BOID,
                new QFilter[]{new QFilter(HBJMConstants.INIT_BATCH, "=", this.getInitInParam().getBatchNumber()),
                        new QFilter(HBJMConstants.INIT_STATUS, "in", new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED})});
        List<Long> idList = (List) Stream.of(dynamicObjects).map((it) -> {
            return it.getLong(HRBaseConstants.ID);
        }).collect(Collectors.toList());
        //删除主实体内容
        hrBaseServiceHelper.delete(idList.toArray());
        //删除事务日志信息
        List<Long> boidList = (List) Stream.of(dynamicObjects).map((it) -> {
            return it.getLong(HisFieldNameConstants.BOID);
        }).collect(Collectors.toList());
        ChangeEventService service = new ChangeEventServiceImpl();
        service.delEvent(boidList);
        this.getInitOutParam().getHandleIdSet().addAll(idList);
    }
}
