package kd.hrmp.hbjm.business.upgrade.impl;

import com.google.common.collect.Lists;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.business.domain.repository.HBJMEventRepository;
import kd.hrmp.hbjm.business.upgrade.IJobEventDataUpgradeService;
import kd.hrmp.hbjm.business.utils.JobMetaDataServiceHlper;
import kd.hrmp.hbjm.common.constants.ChangeEventConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;

import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

public class JobEventDataUpgradeServiceImpl implements IJobEventDataUpgradeService, ChangeEventConstants {

    private Log logger = LogFactory.getLog(JobEventDataUpgradeServiceImpl.class);

    @Override
    public void jobEventDataUpgrade() {
        // 如果不存在hbjm_jobhr元数据，说明是新建库，那这个时候没有存量数据，不需要更新
        if (!JobMetaDataServiceHlper.isMetaDataExist(HBJMConstants.PAGE_JOB)) {
            return;
        }

        DynamicObject[] operationDetailDyns = HBJMEventRepository.getInstance().selectAllOpDetail();
        Map<Long, List<DynamicObject>> jobBoIdVsOperationDetailDynListMap = Arrays.stream(operationDetailDyns)
                .sorted(Comparator.comparing(o -> o.getDate(HRBaseConstants.CREATETIME)))
                .collect(Collectors.groupingBy(dyn -> dyn.getLong(JOB), Collectors.toList()));

        // 存储变动操作明细（hbjm_changeoperdetail）
        List<DynamicObject> changeOperateDetailDynList = Lists.newArrayListWithExpectedSize(operationDetailDyns.length);
        // 存储变动事务明细（hbjm_changetrandetail）
        List<DynamicObject> changeTranDetailDynList = Lists.newArrayListWithExpectedSize(operationDetailDyns.length);
        // 存储变动事务汇总（hbjm_changetransum）
        List<DynamicObject> changeTranSumDynList = Lists.newArrayListWithExpectedSize(operationDetailDyns.length);


        // 1.找到需要升级数据（按时间排序，除第一条之外其他的fbeforejobid为空的）
        Set<Map.Entry<Long, List<DynamicObject>>> jobBoIdVsDynListEntries = jobBoIdVsOperationDetailDynListMap.entrySet();
        for (Map.Entry<Long, List<DynamicObject>> jobBoIdVsDynListEntry : jobBoIdVsDynListEntries) {
            List<DynamicObject> operationDetailDynList = jobBoIdVsDynListEntry.getValue();
            // 第一条应该是新增的，如果不是数据有问题
            DynamicObject preOperationDetailDyn = operationDetailDynList.get(0);
            Long beforeJobIdForPre = preOperationDetailDyn.getLong(BEFORE_JOB);
            Long changeOperateIdForPreDyn = preOperationDetailDyn.getLong(CHANGE_OPERATE_ID);
            if (beforeJobIdForPre != 0L || changeOperateIdForPreDyn != ADD) {
                logger.info("JobEventDataUpgradeServiceImpl jobEventDataUpgrade data error: the first operateDetail data is not add eventId={}, jobBoId = {}",
                        preOperationDetailDyn.getLong(HRBaseConstants.ID), preOperationDetailDyn.getLong("job"));
                continue;
            }

            // 从第二条开始 只要操作类型为新增 并且 beforejob 为空的都是启用导致的
            for (int i = 1; i < operationDetailDynList.size(); i++) {
                DynamicObject operationDetailDyn = operationDetailDynList.get(i);
                Long beforeJobId = operationDetailDyn.getLong(BEFORE_JOB);
                Long changeOperateId = operationDetailDyn.getLong(CHANGE_OPERATE_ID);
                // 启用的导致
                if (beforeJobId == 0L && changeOperateId == ADD) {
                    operationDetailDyn.set(CHANGE_OPERATE, UPDATE);
                    operationDetailDyn.set(BEFORE_JOB, preOperationDetailDyn.getLong(AFTER_JOB));
                    // 塞入 changeOperateDetailDynList
                    changeOperateDetailDynList.add(operationDetailDyn);
                }
                preOperationDetailDyn = operationDetailDyn;
            }
        }

        // 2.拿到feventId之后，找到对应的 变动事务明细（hbjm_changetrandetail） 变动事务汇总（hbjm_changetransum）信息
        Set<Long> eventIdSet = changeOperateDetailDynList.stream()
                .map(dyn -> dyn.getLong(EVENT)).collect(Collectors.toSet());
        DynamicObject[] tranDetailDyns = HBJMEventRepository.getInstance().selectEventDetails(eventIdSet);
        // 更新为变更
        for (DynamicObject tranDetailDyn : tranDetailDyns) {
            tranDetailDyn.set(CHANGE_TYPE, TYPE_UPDATE);
            tranDetailDyn.set(CHANGE_SCENE, SCENE_UPDATE);
            setSubSceneDynCol(tranDetailDyn, SUBSCENE_SCM_CHANGE);
            changeTranDetailDynList.add(tranDetailDyn);
        }
        DynamicObject[] sumDetailDyns = HBJMEventRepository.getInstance().selectEventSumDetails(eventIdSet);
        for (DynamicObject sumDetailDyn : sumDetailDyns) {
            sumDetailDyn.set(ChangeEventConstants.CHANGE_TYPE, TYPE_UPDATE);
            sumDetailDyn.set(ChangeEventConstants.CHANGE_SCENE, SCENE_UPDATE);
            setSubSceneDynCol(sumDetailDyn, SUBSCENE_SCM_CHANGE);
            changeTranSumDynList.add(sumDetailDyn);
        }

        new HRBaseServiceHelper(PAGE_OP_DETAIL).update(changeOperateDetailDynList.toArray(new DynamicObject[0]));
        new HRBaseServiceHelper(PAGE_EVENT_DETAIL).update(changeTranDetailDynList.toArray(new DynamicObject[0]));
        new HRBaseServiceHelper(PAGE_EVENT_SUM).update(changeTranSumDynList.toArray(new DynamicObject[0]));
    }

    private void setSubSceneDynCol(DynamicObject dyn, Long subScene) {
        DynamicObjectCollection collection = new DynamicObjectCollection();
        DynamicObjectType dynamicObjectType = dyn.getDynamicObjectCollection(CHANGE_SUB_SCENE).getDynamicObjectType();
        DynamicObject subcene = new DynamicObject(dynamicObjectType);
        subcene.set("fbasedataid", subScene);
        dyn.set(CHANGE_SUB_SCENE, collection);
    }

}
